/*==============================================================================
*NOME:   REDUZCOR.C
*
*  Projeto: Reduo para 256 cores
*
*  Verso:              V 4.00
*  Autor :   Juliana Carpes Imperial        - 9714169-4
*
*Descrio do mdulo de implementao
*     Objetivo:
*        - Implementar uma funo que aproxima uma dada imagem para uma outra
*          com a mesma resoluo espacial, mas com apenas 256 cores distintas.
*        - Implementar uma funo auxiliar que acha a aproximnao para uma
*          componente pela faixa onde ela se localiza.
*        - Implementar uma funo que dadas as coordenadas de um pixel d as
*          componentes YCrCb dele.
*        - Implementar uma funo que acha o representante de cada faixa das
*          componentes.
*
*Modo de utilizar
*     Ao desenvolver um programa:
*        - Pesquisar as declaraes de tipos em IMAGE.H e IMAGE.C para o
*          correto uso da funo.
*        - Pesquisar em REDUZCOR.H a especificao da funo aqui utilizada
*          para seu correto uso.
*=============================================================================*/

/*==============================================================================
*Verso anterior: V 3.00
*Modificaes: A reduo de cores era por quantizao median cut em RGB e
*              passou a ser median cut em YCrCb.
*=============================================================================*/

/*==============================================================================
*Verso anterior: V 2.00
*Modificaes: A reduo de cores era por quantizao uniforme em YCrCb e passou
*              a ser median cut em RGB.
*=============================================================================*/

/*==============================================================================
*Verso anterior: V 1.00
*Modificaes: A reduo de cores era por quantizao uniforme em RGB e passou a
*              ser por uniforme em YCrCb.
*=============================================================================*/


#include "reduzcor.h"


/*==============================================================================
*Nome: imgReduceTo256
*=============================================================================*/

Image* imgReduceTo256( Image* img ) {

   unsigned char cCorVerm           ,   /* armazena componente vermelha */
                 cCorVerde          ,   /* armazena componente verde */
                 cCorAzul           ,   /* armazena componente azul */
                 cY                 ,   /* armazena componente Y */
                 cCr                ,   /* armazena componente Cr */
                 cCb                ;   /* armazena componente Cb */
   int           i                  ,   /* contador para altura */
                 j                  ;   /* contador para largura */
   long          lNumCores          ,   /* nmero total de cores */
                 lNumY              ,   /* conta componentes Y */
                 lNumCr             ,   /* conta componentes Cr */
                 lNumCb             ;   /* conta componentes Cb */
   tpCor         Y[ I_MAX_CORES ]   ,   /* populao e faixa de Y */
                 Cr[ I_MAX_CORES ]  ,   /* populao e faixa de Cr */
                 Cb[ I_MAX_CORES ]  ;   /* populao e faixa de Cb */

/* inicializa as populaes */
   for ( i = 0 ; i < I_MAX_CORES ; i++ ) {
      Y[ i ].lPopul = 0 ;
      Cr[ i ].lPopul = 0 ;
      Cb[ i ].lPopul = 0 ;
   } /* for */

/* conta as populaes */
   for ( i = 0 ; i < imgGetHeight( img ) ; i++ ) {
      for ( j = 0 ; j < imgGetWidth( img ) ; j++ ) {

      /* Pega componentes em YCrCb do pixel */
         MudarSistema( img, j, i, &cY, &cCr, &cCb ) ;

      /* soma as populaes das componentes YCrCb */
         Y[ cY ].lPopul++ ;
         Cr[ cCr ].lPopul++ ;
         Cb[ cCb ].lPopul++ ;

     } /* for */
   } /* for */

/* populao total de todas as 3 componentes */
   lNumCores = ( long )imgGetHeight( img ) * ( long )imgGetWidth( img ) ;

/* zera populao de vermelho, verde e azul */
   lNumY = lNumCr = lNumCb = 0L ;

/* a faixa inicial de cada componentes  a zero */
   cY = cCb = cCr = 0 ;

   for ( i = 0 ; i < I_MAX_CORES ; i++ ) {

   /* acha faixa do Y */
      AcharFaixa( &lNumY, lNumCores / 8L, Y[ i ].lPopul, &Y[ i ].cFaixa, &cY,
                i ) ;

   /* acha faixa do Cr */
      AcharFaixa( &lNumCr, lNumCores / 8L, Cr[ i ].lPopul, &Cr[ i ].cFaixa,
                &cCr, i ) ;

   /* acha faixa do Cb */
      AcharFaixa( &lNumCb, lNumCores / 4L, Cb[ i ].lPopul, &Cb[ i ].cFaixa,
                &cCb, i ) ;

   } /* for */

/* acha o representante de cada faixa */
   AcharRepr( Y ) ;
   AcharRepr( Cr ) ;
   AcharRepr( Cb ) ;

/* quantiza cada pixel */
   for ( i = 0 ; i < imgGetHeight( img ) ; i++ ) {
      for ( j = 0 ; j < imgGetWidth( img ) ; j++ ) {

      /* Pega componentes em YCrCb do pixel */
         MudarSistema( img, j, i, &cY, &cCr, &cCb ) ;

      /* quantiza as componentes YCrCb */
         cY = Y[ cY ].cFaixa ;
         cCr = Cr[ cCr ].cFaixa ;
         cCb = Cb[ cCb ].cFaixa ;

      /* transforma as novas componentes de YCrCb para RGB */
         cCorVerm = ( unsigned char )( cY + 1.40200 * ( cCr - 128 ) ) ;
         cCorVerde = ( unsigned char )( cY - 0.34414 * ( cCb - 128 )
                                       - 0.71414 * ( cCr-128 ) ) ;
         cCorAzul = ( unsigned char )( cY + 1.77200 * ( cCb - 128 ) ) ;

      /* coloca o novo pixel j quantizado na imagem */
         imgSetPixel( img, j, i, cCorVerm, cCorVerde, cCorAzul ) ;

      } /* for */
   } /* for */

   return img ;

}  /* imgReduceTo256 */


/*==============================================================================
*Nome: AcharFaixa
*=============================================================================*/

void AcharFaixa( long*          lNumCor      ,
                 long           lNumCorFaixa ,
                 long           lPopul       ,
                 unsigned char* cFaixa       ,
                 unsigned char* cCor         ,
                 int            iIndice       ) {

/* A populao de uma faixa  somada a populao da componentes seguinte */
   *lNumCor += lPopul ;

/* testa se a populao atual de uma faixa passou do limite para a populao de
   uma faixa, se passou, tem-se agora uma nova faixa */
   if ( *lNumCor > lNumCorFaixa ) {
      *cCor = ( unsigned char )iIndice ;
      *lNumCor = 0 ;
   } /* if */

/* a nova faixa  a componentes atual */
   *cFaixa = *cCor ;

} /* AcharFaixa */


/*==============================================================================
*Nome: MudarSistema
*=============================================================================*/

void MudarSistema( Image*         pImagem  ,
                   int            ijIndice ,
                   int            iiIndice ,
                   unsigned char* cY       ,
                   unsigned char* cCr      ,
                   unsigned char* cCb       ) {

/* pega as componentes RGB do pixel */
   unsigned char cCorVerm  = imgGetPixelRed( pImagem, ijIndice, iiIndice )   ,
                 cCorVerde = imgGetPixelGreen( pImagem, ijIndice, iiIndice ) ,
                 cCorAzul  = imgGetPixelBlue( pImagem, ijIndice, iiIndice )  ;

/* converte as componentes de RGB para YCrCb */
   *cY = ( unsigned char )( 0.29900 * cCorVerm + 0.58700 * cCorVerde
                           + 0.11400 * cCorAzul ) ;
   *cCb = ( unsigned char )( -0.16874 * cCorVerm - 0.33126 * cCorVerde
                            + 0.50000 * cCorAzul + 128 ) ;
   *cCr = ( unsigned char )( 0.50000 * cCorVerm - 0.41869 * cCorVerde
                            - 0.08131 * cCorAzul + 128 ) ;

} /* MudarSistema */


/*==============================================================================
*Nome: AcharRepr
*=============================================================================*/

void AcharRepr( tpCor Cor[ ] ) {
   int i = 0 , 
	   j = 0 ;
	
/* s termina loop depois de ter passado por todas as cores de uma componente 
   */
	while ( i < I_MAX_CORES ) {
       long          lPopRep = 0L ; /* populao representante da faixa */
       unsigned char cCorRep = i  ; /* faixa representante */

	/* repete enquanto estiver numa determinada faixa e no tiver estourado o
	   vetor */
	   while ( j < I_MAX_CORES 
			&& Cor[ j ].cFaixa == i ) {
	     
	   /* se uma cor  mais populosa que outra, ela representa a faixa */
	      if ( Cor[ j ].lPopul > lPopRep ) {
		     lPopRep = Cor[ j ].lPopul ;
             cCorRep = j ;       
		  } /* if */
              	   
	      j++ ; 
      } /* while */
	  
   /* atualiza representante da faixa */
	  while ( i < j ) {
	     Cor[ i ].cFaixa = cCorRep ;
		 i++ ;
	  } /* while */
     
   } /* while */


} /* AcharRepr */


/* reduzcor.c */
