/*==============================================================================
*NOME:   REDUZCOR.H
*
*  Projeto: Reduo para 256 cores
*
*  Verso:              V 4.00
*  Autor :   Juliana Carpes Imperial        - 9714169-4
*
*Descrio do mdulo de declarao
*     Objetivo:
*        - Definir uma funo que aproxima uma dada imagem para uma outra com a
*          mesma resoluo espacial, mas com apenas 256 cores distintas.
*        - Definir uma funo auxiliar que acha a aproximnao para uma
*          componente pela faixa onde ela se localiza.
*        - Definir uma funo que dadas as coordenadas de um pixel d as
*          componentes YCrCb dele.
*=============================================================================*/

/*==============================================================================
*Verso anterior: V 3.00
*Modificaes: A reduo de cores era por quantizao median cut em RGB e
*              passou a ser median cut em YCrCb.
*=============================================================================*/

/*==============================================================================
*Verso anterior: V 2.00
*Modificaes: A reduo de cores era por quantizao uniforme em YCrCb e passou
*              a ser median cut em RGB.
*=============================================================================*/

/*==============================================================================
*Verso anterior: V 1.00
*Modificaes: A reduo de cores era por quantizao uniforme em RGB e passou a
*              ser por uniforme em YCrCb.
*=============================================================================*/


#if !defined( reduzcor_MOD )
#define       reduzcor_MOD


#include "image.h"

#define I_MAX_CORES 256


/*==============================================================================
*NOME:  tpCor
*Descricao do tipo
*     Representa a populao de uma componente e a nova componente na qual esta
*     ser transformada.
*
*Campos:
*     lPopul - inteiro longo que representa a populao de uma determinada
*              componente;
*     cFaixa - nova componente que esta populao vai passar a ter.
*$.===========================================================================*/

typedef struct {
   long          lPopul ;
   unsigned char cFaixa ;
} tpCor ;


/*==============================================================================
*Nome: imgReduceTo256
*Descrio da Funo: Funo que aproxima uma dada imagem para uma outra com a
*                     mesma resoluo espacial, mas com apenas 256 cores
*                     distintas por quantizao median cut em YCrCb.
*
*Parametro recebido
*   img: ponteiro para a imagem original.
*
*Valor Retornado
*   ponteiro para uma imagem com a mesma resoluo espacial, mas com apenas 256
*   cores distintas.
*=============================================================================*/

Image* imgReduceTo256( Image* img ) ;


/*==============================================================================
*Nome: AcharFaixa
*Descrio da Funo: Funo que acha a componente na qual uma outra vai ser
*                     aproximada na quantizao.
*
*Parametros recebidos
*   lNumCorFaixa - nmero mximo de populao em uma faixa de componentes;
*   lPopul       - populao de uma determinada componente;
*   iIndice      - indice da componente, entre 0 e 255.
*
*Parmetros retornados
*   lNumCor - populao atual em uma faixa;
*   cFaixa  - componente da faixa atual;
*   cCor    - componente desta faixa.
*=============================================================================*/

void AcharFaixa( long*          lNumCor      ,
                 long           lNumCorFaixa ,
                 long           lPopul       ,
                 unsigned char* cFaixa       ,
                 unsigned char* cCor         ,
                 int            iIndice       ) ;


/*==============================================================================
*Nome: MudarSistema
*Descrio da Funo: Funo que pega pixels no sistema RGB e os transforma para
*                     o sistema YCrCb.
*
*Parametros recebidos
*   ijIndice - posio horizontal do pixel na imagem;
*   iiIndice - posio vertical do pixel na imagem;
*   pImagem  - ponteiro para a imagem.
*
*Parmetros retornados
*   cY  - componente Y ;
*   cCr - componente Cr ;
*   cCb - componente Cb.
*=============================================================================*/

void MudarSistema( Image*         pImagem  ,
                   int            ijIndice ,
                   int            iiIndice ,
                   unsigned char* cY       ,
                   unsigned char* cCr      ,
                   unsigned char* cCb       ) ;


/*==============================================================================
*Nome: AcharRepr
*Descrio da Funo: Acha a cor representante de cada faixa de cores.
*
*Parametros recebidos
*   Cor - vetor de populao e faixa de uma componente.
*
*Parmetros retornados
*   Cor - vetor de populao e faixa de uma componente com as faixas 
*         (representantes) atualizados.
*=============================================================================*/

void AcharRepr( tpCor Cor[ ] ) ;


#endif


/* reduzcor.h */